###
#
# Listen is the legal property of mehdi abaakouk <theli48@gmail.com>
# Copyright (c) 2006 Mehdi Abaakouk
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
###


import gst

#import gnomevfswrap
import formats.mpc as mpc


import mutagen.apev2

from mutagen.apev2 import APEv2File
from mutagen.flac import FLAC
from mutagen.id3 import ID3FileType
from mutagen.mp3 import MP3            
from mutagen.oggflac import OggFLAC
from mutagen.oggspeex import OggSpeex
from mutagen.oggtheora import OggTheora
from mutagen.oggvorbis import OggVorbis
from mutagen.trueaudio import TrueAudio
from mutagen.wavpack import WavPack
from easyid3 import EasyID3
from mutagen.m4a import M4A

from song import READ_EXTENTIONS,WRITE_EXTENTIONS
IDS = {
    "title": "title",
    "artist": "artist",
    "album": "album",
    "tracknumber": "#track",
    "genre": "genre",
    "date": "date"
}
tag_convert = {
 ".mpc":{
          "tracknumber":"track",
          "date":"year"
          }
}
def have_mpc():
    try:
        import ctypes
        _libc = ctypes.cdll.LoadLibrary("libc.so.6")
        _mpcdec = ctypes.cdll.LoadLibrary("libmpcdec.so.3")
        return True
    except: return False
    
module_to_try = [
               (MP3,[".mp3",".mp2"],"mad"),
               (OggVorbis,[".ogg"],"vorbis"),
               (OggFLAC,[".oggflac"],"flac"),
               (FLAC,[".flac"],"flac"),
               (OggSpeex,[".spx"],"speex"),
               (TrueAudio,[".tta"],"ttadec"),
               (APEv2File,[".mpc"],"musepack",have_mpc)
               ]
options = []

def File(filename, options=None):
    """
    Grab from mutagen to have the desired behavior
    i want EasyID3 instead of MP3 class
    """
    if options is None:
        options = [MP3, TrueAudio, OggTheora, OggSpeex, OggVorbis, OggFLAC,
                   FLAC, APEv2File, M4A, ID3FileType, WavPack]
                   
    fileobj = file(filename, "rb")
    try:
        header = fileobj.read(128)
        results = [Kind.score(filename, fileobj, header) for Kind in options]
    finally:
        fileobj.close()
    results = zip(results, options)
    results.sort()
    score, Kind = results[-1]
    if score > 0: 
        if Kind == MP3:
            return Kind(filename,ID3=EasyID3)
        else:
            return Kind(filename)
    else: 
        return None


def read(song):
    
    if song.get_protocol()=="file://":
        filename = song.get_path()
    else:
        filename = song.get("uri")
    ext = song.get_ext()
    if ext==".mpc":
        try: audio = APEv2File(filename)
        except mutagen.apev2.APENoHeaderError: audio = {}
    else:
        audio = File(filename,options)
        
    if audio==None: return None
    for file_tag,tag in IDS.iteritems():
        
        
        if tag_convert.has_key(ext) and tag_convert[ext].has_key(file_tag):
            file_tag = tag_convert[ext][file_tag]
            
        if audio.has_key(file_tag) and audio[file_tag]:
            value = audio[file_tag]

            if isinstance(value,list) or song.get_ext()==".mpc":
                value = value[0]
            song[tag] = value
        
    if song.get_ext()==".mpc":
        try: song["#duration"], song["#bitrate"] = mpc.get_info(song)
        except IOError: 
            return False
    else:
        song["#duration"] = int(audio.info.length)*1000
        try: song["#bitrate"] = int(audio.info.bitrate)
        except AttributeError: pass
    return True
    
    
def write(song):
    if song.get_protocol()=="file://":
        filename = song.get_path()
    else:
        filename = song.get("uri")
        
    ext = song.get_ext()
    if ext==".mpc":
        try: audio = mutagen.apev2.APEv2(filename)
        except mutagen.apev2.APENoHeaderError: audio = mutagen.apev2.APEv2()
    else:
        audio = File(filename,options)

    if audio==None: return None
    if audio.tags is None: audio.add_tags()
    for file_tag,tag in IDS.iteritems():

        if tag_convert.has_key(ext) and tag_convert[ext].has_key(file_tag):
            file_tag = tag_convert[ext][file_tag] 

        try:del(audio[file_tag])
        except KeyError:pass
        value = song.get(tag)
        if value!=None and value!="":
            if tag[0] == "#":
                audio[file_tag] = unicode(value)
            else:
                audio[file_tag] = value
    audio.save()
    return True


for mod in module_to_try:
    try: mod[3]
    except IndexError:ok=True
    else: ok= mod[3]()
    if ok and gst.registry_get_default().find_plugin(mod[2]) is not None:
        options.append(mod[0])
        for ext in mod[1]:
            WRITE_EXTENTIONS[ext]= write
            READ_EXTENTIONS[ext]= read
    
